﻿<?php
// Scriptprojekt: IoTControl V1.2
// File: 'index.php
/*	
	Nutzung:
	========
	Script in eine geplante Task einbinden, um z.B. alle 5 Minuten Schaltvorgänge zu prüfen und ggf. auszuführen
	Gerät könnte z.B. eine NAS DS213j sein, auf der im Aufgabenplaner das Script eingebunden ist mit Aufruf: /usr/local/bin/php56 /volume1/web/IoTControl/index.php
	Voraussetzung ist ein PHP ab 5.6 & Mysql Stack zum Test auch XAMPP geeignet.
	

	Zweck:
	========
	Steuert auf Grundlage einer MySQL-Datenbank via Get-Parameter Geräte, die mit der Tasmota-Firmware versehen sind
	Die vorhandenen Tasmota-Geräte mit den Schaltzeiten sind entsprechend in der Datenbank zu hinterlegen!
	
	DB Table 'control'
	Feld            Bedeutung
	--------------------------------------------------------
	id              Identnummer (Autowert)
	aktiv           0/1 bei 1 - Eintrag wird berücksichtigt
	date            Datum einer Ausführung einmalig
	WoTag           Zyklisch nach Wochentagen als binäre Maske Bit 1 für Mo - Bit 7 für So, 0 für jeden Tag. Feld 'WoTag' Nur berücksichtigt wenn Feld 'date' NULL
					SoSaFrDoMiDiMo
					1 0 1 0 1 1 1  bin => 87 dec => Ausführung Mo, Di, Mi, Fr, So
					Mit Montag fängt die Woche an, ist somit die niederwertigste binäre Stelle

	ontime          Auslösezeit der Aktion, bei NULL wird nach Sonnenauf- oder -untergang ausgewertet  
	S_aufuntergang  1 - Sonnenaufgang 0 - Sonnenuntergang
	offset          (-/+) Minuten vor oder nach Sonnenauf- oder -untergang
	
	
	Prozessdaten werden zum Programmlauf gesetzt und sind nicht manuell zu pflegen:
	execute         Timestamp zu dem tatsächlich die Aktion ausgeführt wurde. Bei NULL Ausführung steht noch aus
	                bzw. '0000-00-00 00:00:00' der Zyklus findet heute nicht statt
	realTime        Der aus den Einträgen ermittelte Zeitpunkt, zu dem die Aktion ausgeführt wird
	
	
	DB Table 'device'
	Feld            Bedeutung
	--------------------------------------------------------
	id              Identnummer (Autowert)
	aktiv           1/0 - Gerät komplett aktiv/deaktiv (logisch & verknüpft mit control.aktiv)
	device          Gerätbezeichnung in Kombination der Aktionen, derzeit '_an'/'_aus' implementiert
	
	

*/
require_once('config.inc.php');
require_once('DBConnect.inc.php');
require_once('Functions.php');

echo '<br>IoTControl zum Zeitpunkt: ',date("d:m:Y H:i:s", $TM),'</br>';

//DB-Table 'control' lesen und Zyklen für aktuellen Tag initialisieren
$stm=$Hnd->prepare(
	'SELECT control.id, control.aktiv,date,WoTag,ontime,S_aufuntergang,offset,realTime,device_aktion,execute'
		.' from control LEFT JOIN device'
		.' ON device_id = device.id ORDER BY control.id ASC');
$stm->execute();
echo '<br>';

while ($daten= $stm->fetch(PDO::FETCH_ASSOC)){ //Zyklus für den Tag initialisieren
	print_r($daten);
		if($CNF_TStmpDateNow != strtoTime(date("Y-m-d", strtoTime($daten[execute])))){ //schon einmal heute gelaufen --> übergehen
			if(ZyklenIntervall($daten, $CNF_TStmpDateNow)) //Wenn heute im Zyklus
					ZyklenInitial($Hnd,$daten[id],0,'');   //dann <execute> auf NULL setzen ==> Zyklus für heute aktiv!
			else
					ZyklenInitial($Hnd,$daten[id],1,$CNF_DateTime); //<execute> auf 0 setzen	==> Zyklus für heute inaktiv!
		echo '<br>Es erfolgt eine Zyklusinitialisierung';
		}
	echo '<br>';
}

//Schleife um Feld 'realTime' zu bestimmen nach Sonnenauf- u. -unterang mit Offsetzeit in Minuten
echo '<br>Nach Initialisierung ==> Alles wo execute-time = NULL ist im laufe des Tages auszuführen!';
$stm=$Hnd->prepare("SELECT * FROM control WHERE aktiv=1 ORDER BY id ASC");
$stm->execute();
echo '<br>';
while ($daten= $stm->fetch(PDO::FETCH_ASSOC)){
	//konkrete Zeit für Sonnenauf- u. -untergang ermitteln
	$SoT = date("H:i:s",strtotime($daten[ontime]));
	if(empty($daten[ontime])){
		if($daten[S_aufuntergang]){ // 1 ist Sonnenaufgang
			$SoT = date("H:i",$CNF_Saufg + 60 * $daten[offset]);
		}else{
			$SoT = date("H:i",$CNF_Suntg + 60 * $daten[offset]);
		}
	}
	$stm2=$Hnd->prepare('UPDATE control SET realTime=:dtime WHERE id=:ID');
	$stm2->bindParam(':ID', $daten[id]);
	$stm2->bindParam(':dtime', $SoT);
	$stm2->execute();

}

//DB-Tabelle 'control' mit Einträgen durchlaufen und ggf. Aktionen der Geräte ausführen
$stm=$Hnd->prepare(
	'SELECT control.id, control.aktiv as aktiv,date,WoTag,ontime,S_aufuntergang,offset,realTime,device_aktion,execute,IP_DNS'
		.' from control LEFT JOIN device'
		.' ON device_id = device.id'
		.' WHERE device.aktiv=1 and control.aktiv=1'
		.' ORDER BY realTime ASC');
$stm->execute();
echo '<br>';
while ($daten= $stm->fetch(PDO::FETCH_ASSOC)){ //jeden Eintrag 
	if(empty($daten[execute])){ //nicht NULL ==> Eintrag nicht ausführen
		echo  '<br>Gerät: ',$daten[id],'|',$daten[aktiv],'|',$daten[date],'|',$daten[WoTag],'|',$daten[realTime],'|',$daten[aktion],'|',$daten[device],'|',$daten[execute];

		//Vergleiche mit akt. Zeit
		echo '<br>Db Vergleich:',strtotime($daten[realTime]),'|',$CNF_TStmpTNOW,'<br>'
		,$daten[realTime],'|',date("H:i:s",$CNF_TStmpTNOW);
			
		if(strtotime($daten[realTime]) <= $CNF_TStmpTNOW){
			echo '<br> Kommando aufrufen<br>';
			$RC = IoTSwitch($Hnd, $daten[IP_DNS], $daten[id], $daten[device_aktion], $CNF_DateTime); //Aktion ausführen
		}else
			echo '<br> noch <b>kein</b> Kommando aufrufen<br>';
	}
}


unset($DBHnd);